import { RouteRecordRaw } from "vue-router";
import { defineStore } from "pinia";

import { loginApi } from "@/api/auth";
import { getUserInfoApi } from "@/api/user";
import { constantRoutes, resetRouter } from "@/router";
import { store } from "@/store";

import { LoginData } from "@/api/auth/types";
import { UserInfo } from "@/api/user/types";

import { getToken, setToken, removeToken } from "@/utils/auth";
import { ref } from "vue";
import { isBoolean } from "lodash";
const Layout = () => import("@/layout/index.vue");
const modules = import.meta.glob("../../views/**/**.vue");

type CustomRoute = RouteRecordRaw & {
  hidden?: boolean;
};

export const useUserStore = defineStore("user", () => {
  const user = reactive<UserInfo>({
    token: getToken(),
    username: "",
    avatar: "",
    roles: [],
    permissions: [],
    specialTag: "",
  });
  const menus = ref<RouteRecordRaw[]>([]);
  let menusList: RouteRecordRaw[] = [];

  function setRoutes(newRoutes: RouteRecordRaw[]) {
    menus.value = constantRoutes.concat(newRoutes);
  }
  /**
   * 登录
   *
   * @returns
   */
  function login(loginData: LoginData) {
    return new Promise<void>((resolve, reject) => {
      loginApi(loginData)
        .then((response) => {
          setToken(response.data);
          user.token = response.data; // Bearer eyJhbGciOiJIUzI1NiJ9.xxx.xxx
          resolve();
        })
        .catch((error) => {
          reject(error);
        });
    });
  }

  // 获取信息(用户昵称、头像、角色集合、权限集合)
  function getUserInfo() {
    return new Promise<UserInfo>((resolve, reject) => {
      getUserInfoApi()
        .then(({ data }) => {
          if (data.roles.length > 0) {
            user.roles = data.roles;
          } else {
            user.roles = ["ROLE_DEFAULT"];
          }
          user.permissions = data.permissions;
          user.username = data.user?.username;
          user.specialTag = data.user?.specialTag;
          user.avatar = data.user?.avatar;
          menus.value = data.menus;
          menusList = data.menus;
          resolve(data);
        })
        .catch((error) => {
          reject(error);
        });
    });
  }

  // user logout
  function logout() {
    return new Promise<void>((resolve) => {
      // logoutApi()
      //   .then(() => {
      removeToken();
      user.token = "";
      user.roles = [];
      user.permissions = [];
      location.reload(); // 清空路由
      resolve();
      // })
      // .catch((error) => {
      //   reject(error);
      // });
    });
  }

  // remove token
  function resetToken() {
    return new Promise<void>((resolve) => {
      user.token = "";
      resetRouter();
      resolve();
    });
  }

  /**
   * 生成动态路由
   *
   * @returns
   */
  function generateRoutes() {
    return new Promise<RouteRecordRaw[]>((resolve, reject) => {
      // 接口获取所有路由
      if (menusList.length > 0) {
        // 根据角色获取有访问权限的路由
        const accessedRoutes = filterAsyncRoutes(menusList, 0);
        setRoutes(accessedRoutes);
        resolve(accessedRoutes);
      } else {
        reject();
      }
    });
  }
  function loadView(view: any) {
    // return () => import(`@/views/${view}.vue`);
    return modules[`../../views/${view}.vue`];
  }

  /**
   * 递归过滤有权限的异步(动态)路由
   *
   * @param routes 接口返回的异步(动态)路由
   * @param upIndex 上一个Index
   * @returns 返回用户有权限的异步(动态)路由
   */
  function filterAsyncRoutes(routes: CustomRoute[], upIndex: number) {
    return routes.filter((route, nextIndex) => {
      if (route.path[0] !== "/") {
        route.path = "/" + route.path;
      }
      if (route.meta && isBoolean(route.hidden)) {
        route.meta.hidden = route.hidden;
      }
      if (route.component) {
        // Layout组件特殊处理
        if (route.component?.toString() === "Layout") {
          route.component = Layout;
        } else {
          route.component = loadView(route.component);
        }
      }
      if (route.children != null && route.children && route.children.length) {
        route.children = filterAsyncRoutes(route.children, nextIndex);
      }
      return true;
    });
  }

  return {
    user,
    menus,
    login,
    getUserInfo,
    logout,
    resetToken,
    generateRoutes,
  };
});

// 非setup
export function useUserStoreHook() {
  return useUserStore(store);
}
